<?php

namespace Drupal\node_revisions_autoclean\Commands;

use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\node\Entity\Node;
use Drush\Commands\DrushCommands;

class NodeRevisionsAutocleanCommands extends DrushCommands
{
    use StringTranslationTrait;

    /**
     * Deletes old revisions according to site's settings
     * @command nra-delete-old-revisions
     * @validate-module-enabled node
     * @aliases nra:dor
     */
    public function deleteRevisionsAccordingSiteSettings()
    {
        $nodes = Node::loadMultiple();
        $revisionsManager = \Drupal::service('node_revisions_autoclean.revisions_manager');
        $count = 0;
        foreach ($nodes as $node) {
            $revisions = $revisionsManager->revisionsToDelete($node);
            if (count($revisions)) {
                $revisionsManager->deleteRevisions($revisions);
                $this->logger()->log('success', $this->t('@count revisions deleted for node @nid : @label', [
                    '@count' => count($revisions),
                    '@nid' => $node->id(),
                    '@label' => $node->label(),
                ]));
            }
            $count += count($revisions);
        }
        $this->logger()->log('success', $this->t('Global : @count revisions deleted.', [
            '@count' => $count,
        ]));
    }
}