<?php

namespace Drupal\node_revisions_autoclean\Form;

use Drupal\Core\Entity\EntityTypeBundleInfo;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Class NodeRevisionsByBundleForm.
 */
class NodeRevisionsByBundleForm extends FormBase
{

    /**
     *
     * @var \Drupal\Core\Entity\EntityTypeBundleInfo
     */
    protected $entityTypeBundleInfo;

    /**
     * Constructs a new NodeRevisionsByBundleForm object.
     */
    public function __construct(EntityTypeBundleInfo $entityTypeBundleInfo)
    {
        $this->entityTypeBundleInfo = $entityTypeBundleInfo;
    }

    public static function create(ContainerInterface $container)
    {
        return new static(
            $container->get('entity_type.bundle.info')
        );
    }

    /**
     * {@inheritdoc}
     */
    public function getFormId()
    {
        return 'node_revisions_by_bundle_form';
    }

    /**
     * {@inheritdoc}
     */
    public function buildForm(array $form, FormStateInterface $form_state)
    {
        $types = $this->entityTypeBundleInfo->getBundleInfo('node');
        $config = \Drupal::config('node_revisions_autoclean.settings');

        $form['enable_on_cron'] = [
            '#type' => 'checkbox',
            '#title' => $this->t('Enable old revisions deletion during cronjobs'),
            '#return_value' => '1',
            '#default_value' => $config->get('enable_on_cron') ? $config->get('enable_on_cron') : '0',
            '#description' => $this->t('Cronjobs will delete old revisions according your parameters.')
        ];

        $form['enable_on_node_update'] = [
            '#type' => 'checkbox',
            '#title' => $this->t('Enable old revisions deletion on node update'),
            '#description' => $this->t("Each node's revisions will be autoclean on node update"),
            '#return_value' => '1',
            '#default_value' => $config->get('enable_on_node_update') ? $config->get('enable_on_node_update') : '0',
        ];

        $form['explain'] = [
            '#markup' => '<p><i>' . $this->t('You can select none of the above if you wish to delete old revisions using a drush command (drush nra:dor).') . '</i></p>',
        ];


        foreach ($types as $machine_name => $arr) {
            $form['fs_' . $machine_name] = [
                '#type' => 'fieldset',
                '#title' => $this->t('Content type : @content_type', ['@content_type' => $arr['label']]),
                '#collapsible' => TRUE,
                '#collapsed' => FALSE,
            ];

            $form['fs_' . $machine_name]["node__$machine_name"] = [
                '#type' => 'number',
                '#title' => $this->t('Limit revisions for node type @label', ['@label' => $arr['label']]),
                '#description' => $this->t('Max revisions for @label type, "0" means unlimited number of revisions.', ['@label' => $arr['label']]),
                '#default_value' => $config->get("node.$machine_name") ? $config->get("node.$machine_name") : 0,
                '#required' => true,
            ];
        }
        $form['submit'] = [
            '#type' => 'submit',
            '#value' => $this->t('Submit'),
        ];

        return $form;
    }

    /**
     * {@inheritdoc}
     */
    public function validateForm(array &$form, FormStateInterface $form_state)
    {
        parent::validateForm($form, $form_state);
    }

    /**
     * {@inheritdoc}
     */
    public function submitForm(array &$form, FormStateInterface $form_state)
    {
        $config = \Drupal::configFactory()->getEditable('node_revisions_autoclean.settings');
        $values = $form_state->getValues();

        $enable_on_cron = (int)$form_state->getValue('enable_on_cron');
        $config->set('enable_on_cron', $enable_on_cron);

        $enable_on_node_update = (int)$form_state->getValue('enable_on_node_update');
        $config->set('enable_on_node_update', $enable_on_node_update);

        foreach ($values as $key => $val) {
            if (strpos($key, 'node__') === 0) {
                $key = str_replace('__', '.', $key);
                $config->set($key, (int)$val);
            }
        }
        $config->save(true);
        drupal_set_message($this->t('Node revisions settings have been updated.'));
    }

}
