<?php

namespace Drupal\node_revisions_autoclean\Services;

use Drupal\Core\Entity\EntityTypeManager;
use Drupal\Core\Queue\QueueFactory;
use Drupal\node\Entity\Node;

/**
 * Class RevisionsManager.
 */
class RevisionsManager
{
    /**
     * Drupal\Core\Entity\EntityTypeManager definition.
     *
     * @var \Drupal\Core\Entity\EntityTypeManager
     */
    protected $entityTypeManager;

    /**
     * Drupal\Core\Queue\QueueFactory definition.
     *
     * @var QueueFactory
     */
    protected $queueFactory;

    /**
     * Constructs a new RevisionsManager object.
     */
    public function __construct(EntityTypeManager $entity_type_manager, QueueFactory $queueFactory)
    {
        $this->entityTypeManager = $entity_type_manager;
        $this->queueFactory = $queueFactory;
    }

    /**
     * @param Node $entity
     * @return Node[]
     */
    public function loadRevisions(Node $entity)
    {
        $vids = $this->entityTypeManager->getStorage('node')->revisionIds($entity);
        $revisions = [];
        foreach ($vids as $vid) {
            $revisions[] = $this->entityTypeManager->getStorage('node')->loadRevision($vid);
        }
        return $revisions;
    }

    /**
     * @param Node[] $nodes
     * @return Node[]
     */
    public function revisionsToDelete(Node $node, array $revisions = [])
    {
        $settings = \Drupal::config('node_revisions_autoclean.settings');
        $max = $settings->get('node.' . $node->bundle());
        $result = [];
        if ($max) {
            if (!count($revisions)) {
                $revisions = $this->loadRevisions($node);
            }
            $revisions = array_reverse($revisions);
            $count = 0;
            $bStart = false;
            /* @var $revision Node */
            foreach ($revisions as $revision) {
                if ($bStart) {
                    $count++;
                }
                if ($revision->get('status')->value) {
                    $bStart = true;
                }
                if ($count > $max) {
                    $result[] = $revision->vid->value;
                }
            }
        }
        return $result;
    }

    /**
     * @param $revisionID
     */
    public function deleteRevision($revisionID)
    {
        $this->deleteRevisions([$revisionID]);
    }

    /**
     * @param int[] $revisionsIDs
     */
    public function deleteRevisions(array $revisionsIDs)
    {
        foreach ($revisionsIDs as $revisionID) {
            $this->entityTypeManager->getStorage('node')->deleteRevision($revisionID);
        }
    }

    public function queueNodeForCronJob(Node $node)
    {
        $queue = $this->queueFactory->get('cleanup_revisions_worker');
        $queue->createItem((object)[
            'node' => $node
        ]);
    }

}
